searchData={"items":[{"type":"module","title":"MDEx","doc":"A CommonMark-compliant fast and extensible Markdown parser and formatter for Elixir.","ref":"MDEx.html"},{"type":"module","title":"Features - MDEx","doc":"Compliant with [CommonMark](https://spec.commonmark.org) and [GitHub Flavored Markdown](https://github.github.com/gfm) specifications with extra [extensions](https://docs.rs/comrak/latest/comrak/struct.ExtensionOptions.html)\nas Wiki Links, Discord Markdown tags, and emoji. Also supports syntax highlighting out-of-the-box using the [Autumn](https://github.com/leandrocp/autumn) library.\n\nUnder the hood it's calling the [comrak](https://crates.io/crates/comrak) APIs to process Markdown,\na fast Rust crate that ports the cmark fork maintained by GitHub, a widely and well adopted Markdown implementation.\n\nThe AST format is based on [Floki](https://hex.pm/packages/floki) so the same API to manipulate HTML can be used to manipulate Markdown documents.\nCheck out some examples at [mdex/examples/](https://github.com/leandrocp/mdex/tree/main/examples)\n\nAnd some samples are available at https://mdex-c31.pages.dev","ref":"MDEx.html#module-features"},{"type":"module","title":"Installation - MDEx","doc":"Add `:mdex` dependency:\n\n```elixir\ndef deps do\n  [\n    {:mdex, \"~> 0.2\"}\n  ]\nend\n```","ref":"MDEx.html#module-installation"},{"type":"module","title":"Usage - MDEx","doc":"```elixir\nMix.install([{:mdex, \"~> 0.2\"}])\n```\n\n```elixir\nMDEx.to_html!(\"# Hello\")\n\" Hello \\n\"\n```\n\n```elixir\nMDEx.to_html!(\"# Hello :smile:\", extension: [shortcodes: true])\n\" Hello 😄 \\n\"\n```","ref":"MDEx.html#module-usage"},{"type":"module","title":"Sigils - MDEx","doc":"Convert between Markdown, HTML, and AST.\n\n```elixir\nimport MDEx.Sigil\n```\n\n```elixir\n~M|# Hello from `~M` sigil|\n\" Hello from  ~M  sigil \\n\"\n```\n\n```elixir\n~M|`~M` can return the AST too|AST\n[\n  {\"document\", [], [{\"paragraph\", [], [{\"code\", [{\"num_backticks\", 1}, {\"literal\", \"~M\"}], []}, \" can return the AST too\"]}]}\n]\n\" Hello \\n\"\n```\n\n```elixir\ntitle = \"Hello from variable\"\n\n~m|[{\"document\", [], [{\"heading\", [], [\"#{title}\"]}]}]|\n\" Hello from variable \\n\"\n```\n\nSee all modifiers and examples at https://hexdocs.pm/mdex/MDEx.Sigil.html","ref":"MDEx.html#module-sigils"},{"type":"module","title":"Safety - MDEx","doc":"For security reasons, every piece of raw HTML is omitted from the output by default:\n\n```elixir\nMDEx.to_html!(\" Hello \")\n\"<!-- raw HTML omitted -->\\n\"\n```\n\nThat's not very useful for most cases, so you can render raw HTML but escaping it for safety:\n\n```elixir\nMDEx.to_html!(\" Hello \", render: [escape: true])\n\"&lt;h1&gt;Hello&lt;/h1&gt;\\n\"\n```\n\nIf the input is provided by external sources, it might be a good idea to sanitize it instead for extra security:\n\n```elixir\nMDEx.to_html!(\" Elixir \", render: [unsafe_: true], features: [sanitize: true])\n\"  Elixir  \\n\"\n```\n\nNote that you must pass the `unsafe_: true` option to first generate the raw HTML in order to sanitize it.\n\nAll sanization rules are defined in the [ammonia docs](https://docs.rs/ammonia/latest/ammonia/fn.clean.html).\nFor example, the link in the example below was marked as `noopener noreferrer` to prevent attacks.\n\nIf those rules are too strict and you really trust the input, or you really need to render raw HTML,\nthen you can just render it directly without escaping nor sanitizing:\n\n```elixir\nMDEx.to_html!(\" alert('hello') \", render: [unsafe_: true])\n\" alert('hello') \\n\"\n```","ref":"MDEx.html#module-safety"},{"type":"module","title":"Parsing - MDEx","doc":"Converts Markdown to an AST data structure that can be inspected and manipulated to change the content of the document.\n\nThe data structure shape is exactly the same as the one used by [Floki](https://github.com/philss/floki) so we can reuse the same APIs and keep the same mental model when\nworking with these documents, either Markdown or HTML, where each node is represented as:\n\n```elixir\n{name, attributes, children}\n```\n\nExample:\n\n```elixir\nMDEx.parse_document!(\"# Hello\")\n[{\"document\", [], [{\"heading\", [{\"level\", 1}, {\"setext\", false}], [\"Hello\"]}]}]\n```\n\nNote that text nodes have no attributes nor children, so it's represented as a string inside a list.\n\nYou can find the full AST spec on the MDEx module types section.","ref":"MDEx.html#module-parsing"},{"type":"module","title":"Formatting - MDEx","doc":"Converts the AST to a human-readable document, most commonly to HTML, example:\n\n```elixir\nMDEx.to_html!([{\"document\", [], [{\"heading\", [{\"level\", 1}, {\"setext\", false}], [\"Hello\"]}]}])\n\" Hello \\n\"\n```\n\n_More formats can be added in the future._\n\nAny missing attribute will be filled with the default value, and extra attributes will be ignored. So you could have the same result with:\n\n```elixir\nMDEx.to_html!([{\"document\", [], [{\"heading\", [], [\"Hello\"]}]}])\n\" Hello \\n\"\n```\n\nDefault values are defined on a best-case scenario but as a good practice you should provide all attributes for each node.\n\nTrying to format malformed ASTs will return a `{:error, %DecodeError{}}` describing what and where the error occurred, for example:\n\n```elixir\n{:error, decode_error} = MDEx.to_html([{\"code\", [{1, \"foo\"}], []}], [])\n{:error,\n %MDEx.DecodeError{\n   reason: :attr_key_not_string,\n   found: \"1\",\n   node: \"(<<\\\"code\\\">>, [{1,<<\\\"foo\\\">>}], [])\",\n   attr: \"(1, <<\\\"foo\\\">>)\",\n   kind: \"Integer\"\n }}\n\ndecode_error |> Exception.message() |> IO.puts()\n# invalid attribute key\n#\n# Expected an attribute key encoded as UTF-8 binary\n#\n# Got:\n#\n#   1\n#\n# Type:\n#\n#   Integer\n#\n# In this node:\n#\n#   (<<\"code\">>, [{1,<<\"foo\">>}], [])\n#\n# In this attribute:\n#\n#   (1, <<\"foo\">>)\n```","ref":"MDEx.html#module-formatting"},{"type":"module","title":"Options - MDEx","doc":"You can enable extensions and change the output of the generated Markdown by passing any of the available [Comrak Options](https://docs.rs/comrak/latest/comrak/struct.Options.html)\nas keyword lists or also an additional `:features` option.\n\n_The full documentation and list of all options with description and examples can be found on the links below:_\n\n* `:extension` - https://docs.rs/comrak/latest/comrak/struct.ExtensionOptions.html\n* `:parse` - https://docs.rs/comrak/latest/comrak/struct.ParseOptions.html\n* `:render` - https://docs.rs/comrak/latest/comrak/struct.RenderOptions.html\n* `:features` - see the available options below","ref":"MDEx.html#module-options"},{"type":"module","title":"Features Options - MDEx","doc":"* `:sanitize` (defaults to `false`) - sanitize output using [ammonia](https://crates.io/crates/ammonia). See the [Safety](#module-safety) section for more info.\n* `:syntax_highlight_theme` (defaults to `\"onedark\"`) - syntax highlight code fences using [autumn themes](https://github.com/leandrocp/autumn/tree/main/priv/themes),\nyou should pass the filename without special chars and without extension, for example you should pass `syntax_highlight_theme: \"adwaita_dark\"` to use the [Adwaita Dark](https://github.com/leandrocp/autumn/blob/main/priv/themes/adwaita-dark.toml) theme\n* `:syntax_highlight_inline_style` (defaults to `true`) - embed styles in the output for each generated token. You'll need to [serve CSS themes](https://github.com/leandrocp/autumn?tab=readme-ov-file#linked) if inline styles are disabled to properly highlight code\n\nSee some examples below on how to use the provided options:","ref":"MDEx.html#module-features-options"},{"type":"module","title":"GitHub Flavored Markdown with [emojis](https://www.webfx.com/tools/emoji-cheat-sheet/) - MDEx","doc":"```elixir\nMDEx.to_html!(\n  ~S\"\"\"\n  # GitHub Flavored Markdown :rocket:\n\n  - [x] Task A\n  - [x] Task B\n  - [ ] Task C\n\n  | Feature | Status |\n  | ------- | ------ |\n  | Fast | :white_check_mark: |\n  | GFM  | :white_check_mark: |\n\n  Check out the spec at https://github.github.com/gfm/\n  \"\"\",\n  extension: [\n    strikethrough: true,\n    tagfilter: true,\n    table: true,\n    autolink: true,\n    tasklist: true,\n    footnotes: true,\n    shortcodes: true,\n  ],\n  parse: [\n    smart: true,\n    relaxed_tasklist_matching: true,\n    relaxed_autolinks: true\n  ],\n  render: [\n     github_pre_lang: true,\n     escape: true\n  ]\n) |> IO.puts()\n#  GitHub Flavored Markdown 🚀 \n#  \n#      Task A \n#      Task B \n#      Task C \n#  \n#  \n#    \n#      \n#        Feature \n#        Status \n#      \n#    \n#    \n#      \n#        Fast \n#        ✅ \n#      \n#      \n#        GFM \n#        ✅ \n#      \n#    \n#  \n#  Check out the spec at  https://github.github.com/gfm/  \n```","ref":"MDEx.html#module-github-flavored-markdown-with-emojis-https-www-webfx-com-tools-emoji-cheat-sheet"},{"type":"module","title":"Code Syntax Highlighting - MDEx","doc":"````elixir\nMDEx.to_html!(~S\"\"\"\n```elixir\nString.upcase(\"elixir\")\n```\n\"\"\",\nfeatures: [syntax_highlight_theme: \"catppuccin_latte\"]\n) |> IO.puts()\n#  \n#    \n#      String  .  upcase  (  &quot;elixir&quot;  ) \n#    \n#  \n````","ref":"MDEx.html#module-code-syntax-highlighting"},{"type":"module","title":"Demo and Samples - MDEx","doc":"A [livebook](https://github.com/leandrocp/mdex/blob/main/playground.livemd) and a [script](https://github.com/leandrocp/mdex/blob/main/playground.exs) are available to play with and experiment with this library, or you can check out all [available samples](https://github.com/leandrocp/mdex/tree/main/priv/generated/samples) at https://mdex-c31.pages.dev","ref":"MDEx.html#module-demo-and-samples"},{"type":"module","title":"Used By - MDEx","doc":"- [BeaconCMS](https://github.com/BeaconCMS/beacon)\n- [Tableau](https://github.com/elixir-tools/tableau)\n- And [more...](https://github.com/search?q=lang%3Aelixir+%3Amdex&type=code)\n\n_Are you using MDEx and want to list your project here? Please send a PR!_","ref":"MDEx.html#module-used-by"},{"type":"module","title":"Benchmark - MDEx","doc":"A [simple script](benchmark.exs) is available to compare existing libs:\n\n```\nName              ips        average  deviation         median         99th %\ncmark         22.82 K      0.0438 ms    ±16.24%      0.0429 ms      0.0598 ms\nmdex           3.57 K        0.28 ms     ±9.79%        0.28 ms        0.33 ms\nmd             0.34 K        2.95 ms    ±10.56%        2.90 ms        3.62 ms\nearmark        0.25 K        4.04 ms     ±4.50%        4.00 ms        4.44 ms\n\nComparison:\ncmark         22.82 K\nmdex           3.57 K - 6.39x slower +0.24 ms\nmd             0.34 K - 67.25x slower +2.90 ms\nearmark        0.25 K - 92.19x slower +4.00 ms\n```","ref":"MDEx.html#module-benchmark"},{"type":"module","title":"Motivation - MDEx","doc":"MDEx was born out of the necessity of parsing CommonMark files, to parse hundreds of files quickly, and to be easily extensible by consumer of the library.\n\n* [earmark](https://hex.pm/packages/earmark) is extensible but [can't parse](https://github.com/RobertDober/earmark_parser/issues/126) all kinds of documents and is slow to convert hundreds of markdowns.\n* [md](https://hex.pm/packages/md) is very extensible but the doc says \"If one needs to perfectly parse the common markdown, Md is probably not the correct choice\" and CommonMark was a requirement to parse many existing files.\n* [markdown](https://hex.pm/packages/markdown) is not precompiled and has not received updates in a while.\n* [cmark](https://hex.pm/packages/cmark) is a fast CommonMark parser but it requires compiling the C library, is hard to extend, and was archived on Apr 2024\n\n_Note that MDEx is the only one that syntax highlights out-of-the-box which contributes to make it slower than cmark._\n\nTo finish, a friendly reminder that all libs have their own strengths and trade-offs so use the one that better suit your needs.","ref":"MDEx.html#module-motivation"},{"type":"module","title":"Looking for help with your Elixir project? - MDEx","doc":"At DockYard we are [ready to help you build your next Elixir project](https://dockyard.com/phoenix-consulting).\nWe have a unique expertise in Elixir and Phoenix development that is unmatched and we love to [write about Elixir](https://dockyard.com/blog/categories/elixir).\n\nHave a project in mind? [Get in touch](https://dockyard.com/contact/hire-us)!","ref":"MDEx.html#module-looking-for-help-with-your-elixir-project"},{"type":"module","title":"Acknowledgements - MDEx","doc":"* [comrak](https://crates.io/crates/comrak) crate for all the heavy work on parsing Markdown and rendering HTML\n* [Floki](https://hex.pm/packages/floki) for the AST manipulation\n* [Logo](https://www.flaticon.com/free-icons/rpg) created by Freepik - Flaticon\n* [Logo font](https://github.com/quoteunquoteapps/CourierPrime) designed by [Alan Greene](https://github.com/a-dg)","ref":"MDEx.html#module-acknowledgements"},{"type":"function","title":"MDEx.attribute/2","doc":"Returns a list with attribute values for a given `attribute_name`, otherwise returns an empty list.","ref":"MDEx.html#attribute/2"},{"type":"function","title":"Example - MDEx.attribute/2","doc":"iex> MDEx.attribute({\n    ...>   \"code_block\",\n    ...>   [{\"info\", \"mermaid\"}, {\"literal\", \"graph TD;\\n  A-->B;\"}],\n    ...>   []\n    ...> }, \"literal\")\n    [\"graph TD;\\n  A-->B;\"]\n\n    iex> MDEx.attribute({\n    ...>   \"code_block\",\n    ...>   [{\"info\", \"mermaid\"}, {\"literal\", \"graph TD;\\n  A-->B;\"}],\n    ...>   []\n    ...> }, \"other\")\n    []","ref":"MDEx.html#attribute/2-example"},{"type":"function","title":"MDEx.parse_document/2","doc":"Parse markdown and returns the AST.","ref":"MDEx.html#parse_document/2"},{"type":"function","title":"Options - MDEx.parse_document/2","doc":"See the [Options](#module-options) section for the available options.","ref":"MDEx.html#parse_document/2-options"},{"type":"function","title":"Examples - MDEx.parse_document/2","doc":"iex> MDEx.parse_document!(\"# Languages\\n Elixir and Rust\")\n    [\n      {\"document\", [], [\n        {\"heading\", [{\"level\", 1}, {\"setext\", false}], [\"Languages\"]},\n        {\"paragraph\", [], [\"Elixir and Rust\"]}\n      ]}\n    ]\n\n    iex> MDEx.parse_document!(\"Darth Vader is ||Luke's father||\", extension: [spoiler: true])\n    [\n      {\"document\", [], [\n        {\"paragraph\", [], [\n          \"Darth Vader is \",\n          {\"spoilered_text\", [], [\"Luke's father\"]}\n      ]}]}\n    ]","ref":"MDEx.html#parse_document/2-examples"},{"type":"function","title":"MDEx.parse_document!/2","doc":"Same as `parse_document/2` but raises if the parsing fails.","ref":"MDEx.html#parse_document!/2"},{"type":"function","title":"MDEx.to_commonmark/1","doc":"Convert an AST to CommonMark using default options.\n\nTo customize the output, use `to_commonmark/2`.","ref":"MDEx.html#to_commonmark/1"},{"type":"function","title":"Example - MDEx.to_commonmark/1","doc":"iex> MDEx.to_commonmark([{\"document\", [], [{\"heading\", [{\"level\", 3}], [\"Hello\"]}]}])\n    {:ok, \"","ref":"MDEx.html#to_commonmark/1-example"},{"type":"function","title":"Hello\\n\"} - MDEx.to_commonmark/1","doc":"","ref":"MDEx.html#to_commonmark/1-hello-n"},{"type":"function","title":"MDEx.to_commonmark/2","doc":"Convert an AST to CommonMark with custom options.","ref":"MDEx.html#to_commonmark/2"},{"type":"function","title":"Options - MDEx.to_commonmark/2","doc":"See the [Options](#module-options) section for the available options.","ref":"MDEx.html#to_commonmark/2-options"},{"type":"function","title":"MDEx.to_commonmark!/1","doc":"Same as `to_commonmark/1` but raises `MDEx.DecodeError` if the conversion fails.","ref":"MDEx.html#to_commonmark!/1"},{"type":"function","title":"Example - MDEx.to_commonmark!/1","doc":"iex> MDEx.to_commonmark([{\"document\", [], [{\"heading\", [{}], [\"Hello\"]}]}])\n    {:error,\n     %MDEx.DecodeError{\n       reason: :missing_attr_field,\n       found: \"[{}]\",\n       node: \"(<<\\\"heading\\\">>, [{}], [<<\\\"Hello\\\">>])\",\n       attr: nil,\n       kind: nil\n     }}","ref":"MDEx.html#to_commonmark!/1-example"},{"type":"function","title":"MDEx.to_commonmark!/2","doc":"Same as `to_commonmark/2` but raises `MDEx.DecodeError` if the conversion fails.","ref":"MDEx.html#to_commonmark!/2"},{"type":"function","title":"MDEx.to_html/1","doc":"Convert either markdown or an AST to HTML using default options.\n\nTo customize the output, use `to_html/2`.","ref":"MDEx.html#to_html/1"},{"type":"function","title":"Examples - MDEx.to_html/1","doc":"iex> MDEx.to_html(\"# MDEx\")\n    {:ok, \" MDEx \\n\"}\n\n    iex> MDEx.to_html(\"Implemented with:\\n1. Elixir\\n2. Rust\")\n    {:ok, \" Implemented with: \\n \\n Elixir \\n Rust \\n \\n\"}\n\n    iex> MDEx.to_html([{\"document\", [], [{\"heading\", [{\"level\", 3}], [\"MDEx\"]}]}])\n    {:ok, \" MDEx \\n\"}","ref":"MDEx.html#to_html/1-examples"},{"type":"function","title":"MDEx.to_html/2","doc":"Convert `markdown` to HTML with custom `opts`.","ref":"MDEx.html#to_html/2"},{"type":"function","title":"Options - MDEx.to_html/2","doc":"See the [Options](#module-options) section for the available options.","ref":"MDEx.html#to_html/2-options"},{"type":"function","title":"Examples - MDEx.to_html/2","doc":"iex> MDEx.to_html(\"Hello ~world~ there\", extension: [strikethrough: true])\n    {:ok, \" Hello  world  there \\n\"}\n\n    iex> MDEx.to_html(\" visit https://https://beaconcms.org \", extension: [autolink: true], render: [unsafe_: true])\n    {:ok, \"  visit  https://https://beaconcms.org   \\n\"}","ref":"MDEx.html#to_html/2-examples"},{"type":"function","title":"MDEx.to_html!/1","doc":"Same as `to_html/1` but raises `MDEx.DecodeError` if the conversion fails.","ref":"MDEx.html#to_html!/1"},{"type":"function","title":"MDEx.to_html!/2","doc":"Same as `to_html/2` but raises `MDEx.DecodeError` if the conversion fails.","ref":"MDEx.html#to_html!/2"},{"type":"function","title":"MDEx.traverse_and_update/2","doc":"Traverses and updates a Markdown AST.","ref":"MDEx.html#traverse_and_update/2"},{"type":"function","title":"Example - MDEx.traverse_and_update/2","doc":"iex> ast = [{\"document\", [], [{\"heading\", [{\"level\", 1}, {\"setext\", false}], [\"Hello\"]}]}]\n    iex> MDEx.traverse_and_update(ast, fn\n    ...>   {\"heading\", _attrs, children} -> {\"heading\", [{\"level\", 2}], children}\n    ...>   other -> other\n    ...> end)\n    [{\"document\", [], [{\"heading\", [{\"level\", 2}], [\"Hello\"]}]}]\n\nSee more on the [examples](https://github.com/leandrocp/mdex/tree/main/examples) directory.","ref":"MDEx.html#traverse_and_update/2-example"},{"type":"type","title":"MDEx.md_ast/0","doc":"The AST (Abstract Syntax Tree) representation of a Markdown document.\n\nIt's composed by a list of nodes starting with the `document` root node.","ref":"MDEx.html#t:md_ast/0"},{"type":"type","title":"Example - MDEx.md_ast/0","doc":"[\n      {\"document\", [], [\n        {\"heading\", [{\"level\", 1}], [\"Elixir\"]}\n      ]}\n    ]\n\nSee `t:md_node/0` for more info.","ref":"MDEx.html#t:md_ast/0-example"},{"type":"type","title":"MDEx.md_attribute/0","doc":"Attributes of node elements are key-value pairs where key is always a string and value can be of multiple different types.","ref":"MDEx.html#t:md_attribute/0"},{"type":"type","title":"Examples - MDEx.md_attribute/0","doc":"{\"level\", 1}\n    {\"delimiter\", \"period\"}","ref":"MDEx.html#t:md_attribute/0-examples"},{"type":"type","title":"MDEx.md_element/0","doc":"Elements are composed by a name, a list of attributes and a list of children.","ref":"MDEx.html#t:md_element/0"},{"type":"type","title":"Example - MDEx.md_element/0","doc":"{\"heading\", [{\"level\", 1}, {\"setext\", false}], children}","ref":"MDEx.html#t:md_element/0-example"},{"type":"type","title":"MDEx.md_node/0","doc":"Each node of the AST document.\n\nRepresented either as a tuple or a string.","ref":"MDEx.html#t:md_node/0"},{"type":"type","title":"MDEx.md_text/0","doc":"Text element. It has no attributes or children so it's represented just as a string.","ref":"MDEx.html#t:md_text/0"},{"type":"module","title":"MDEx.Sigil","doc":"Sigils for parsing and formatting Markdown.","ref":"MDEx.Sigil.html"},{"type":"module","title":"Modifiers - MDEx.Sigil","doc":"* `AST` - converts Markdown to AST, equivalent to calling `MDEx.parse_document!/2`\n  * `MD` - converts AST to Markdown, equivalent to calling `MDEx.to_commonmark!/2`\n\nWithout a modifier it converts the input (either Markdown or AST) to HTML, equivalent to calling `MDEx.to_html!/2`.\n\nNote that you should `import MDEx.Sigil` to use the `~M` or `~m` sigils.","ref":"MDEx.Sigil.html#module-modifiers"},{"type":"module","title":"Options - MDEx.Sigil","doc":"In order to support the most common scenarios, the sigils have the following options enabled by default:\n\n    [extension: [strikethrough: true, table: true, autolink: true, tasklist: true, superscript: true, footnotes: true, description_lists: true, multiline_block_quotes: true, math_dollars: true, math_code: true, shortcodes: true, underline: true, spoiler: true], parse: [smart: true, relaxed_tasklist_matching: true, relaxed_autolinks: true], render: [_unsafe: true, escape: true]]\n\nIf you need a different set of options, you can call the regular functions like `MDEx.to_html/2` to pass the options you need.","ref":"MDEx.Sigil.html#module-options"},{"type":"macro","title":"MDEx.Sigil.sigil_M/2","doc":"The `~M` sigil converts between CommonMark, HTML, and AST without interpolation.","ref":"MDEx.Sigil.html#sigil_M/2"},{"type":"macro","title":"Examples - MDEx.Sigil.sigil_M/2","doc":"# markdown to html\n    iex> ~M|# Hello|\n    \" Hello \n\"\n\n    # markdown to ast\n    iex> ~M|# Hello|AST\n    [{\"document\", [], [{\"heading\", [{\"level\", 1}, {\"setext\", false}], [\"Hello\"]}]}]\n\n    # ast to markdown\n    iex> ~M|[{\"document\", [], [{\"heading\", [{\"level\", 1}], [\"Hello\"]}]}]|MD\n    \"# Hello\n\"\n\n    # ast to html\n    iex> ~M|[{\"document\", [], [{\"heading\", [{\"level\", 1}], [\"Hello\"]}]}]|\n    \" Hello \n\"","ref":"MDEx.Sigil.html#sigil_M/2-examples"},{"type":"macro","title":"MDEx.Sigil.sigil_m/2","doc":"The `~m` sigil converts between CommonMark, HTML, and AST with interpolation.","ref":"MDEx.Sigil.html#sigil_m/2"},{"type":"macro","title":"Examples - MDEx.Sigil.sigil_m/2","doc":"iex> lang = \"elixir\"\n\n    # markdown to html\n    iex> ~m|`lang = #{lang}`|\n    \"  lang = elixir  \n\"\n\n    # markdown to ast\n    iex> ~m|`lang = #{lang}`|AST\n    [{\"document\", [], [{\"paragraph\", [], [{\"code\", [{\"num_backticks\", 1}, {\"literal\", \"lang = elixir\"}], []}]}]}]\n\n    # ast to markdown\n    iex> ~m|[{\"document\", [], [{\"paragraph\", [], [{\"code\", [{\"num_backticks\", 1}, {\"literal\", \"lang = #{lang}\"}], []}]}]}]|MD\n    \"`lang = elixir`\n\"\n\n    # ast to html\n    iex> ~m|[{\"document\", [], [{\"paragraph\", [], [{\"code\", [{\"num_backticks\", 1}, {\"literal\", \"lang = #{lang}\"}], []}]}]}]|\n    \"  lang = elixir  \n\"","ref":"MDEx.Sigil.html#sigil_m/2-examples"},{"type":"exception","title":"MDEx.DecodeError","doc":"Represents an error decoding the AST from the Elixir data structure into Rust.\n\nThis module is defined as an exception so a message can be generated calling `Exception.message/1`\nor it can be raised.","ref":"MDEx.DecodeError.html"},{"type":"exception","title":"Fields - MDEx.DecodeError","doc":"* `:reason` - an atom representing the type of decode error:\n\n      * `:invalid_structure` - AST structure is invalid or malformed.\n\n      * `:empty` - AST is empty, no nodes found.\n\n      * `:missing_node_field` - one or more fields of a node is missing, it should contain `{name, attributes, children}`.\n\n      * `:missing_attr_field` - either the key or value of an attribute is missing.\n\n      * `:node_name_not_string` - node name is not a UTF-8 encoded string.\n\n      * `:unknown_node_name` - node name is not one of the available node names.\n\n      * `:attr_key_not_string` - attribute key is not a UTF-8 encoded string.\n\n      * `:unknown_attr_value` - attribute value doesn't match any expected type.\n\n  * `:found` - the source that caused the error, either the node or attribute.\n      Most of the times it's displayed as the raw [Term](https://docs.rs/rustler/latest/rustler/struct.Term.html) value as a string (debug).\n      That's because the error may be caused by malformed or unexpected data and we can't decode it to a human-readable format.\n\n  * `:node` - the node where the error was found, usually when the error is in an attribute or children.\n\n  * `:attr` - the attribute where the error was found.\n\n  * `:kind` - the type of the field that caused the error.","ref":"MDEx.DecodeError.html#module-fields"},{"type":"function","title":"MDEx.DecodeError.message/1","doc":"","ref":"MDEx.DecodeError.html#message/1"},{"type":"type","title":"MDEx.DecodeError.t/0","doc":"","ref":"MDEx.DecodeError.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"0.2.0 (2024-10-09) - Changelog","doc":"","ref":"changelog.html#0-2-0-2024-10-09"},{"type":"extras","title":"Breaking changes - Changelog","doc":"* `to_html/1` and `to_html/2` now returns `{:ok, String.t()}` or `{:error, %MDEx.DecodeError{}}` instead of just `String.t()`.\n    The reason is because now they may accept an AST as input which may cause decoding errors.\n    Replace with `to_html!/1` and `to_html!/2` to have the same behavior as before.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix misspelling of `thematic` causing render errors - [#73](https://github.com/leandrocp/mdex/pull/73) by @jonklein","ref":"changelog.html#fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added `to_commonmark/1` and `to_commonmark/2` to convert an AST to CommonMark - [#70](https://github.com/leandrocp/mdex/pull/70) by @jonklein\n  * Added `~M` sigil (no interpolation) with `AST` and `MD` modifiers (defaults to HTML without the modifier)\n  * Added `~m` sigil (supports interpolation and escaping) with `AST` and `MD` modifiers (defaults to HTML without the modifier)\n  * Added `parse_document/1` and `parse_document/2` to parse Markdown to AST\n  * Added low-level functions `traverse_and_update/2` and `attribute/2` to manipulate AST\n  * Added `to_html!/1` and `to_html!/2`, the raising version of `to_html/1` and `to_html/2` (similar to previous `to_html/1` and `to_html/2`)\n  * Changed `to_html/1` and `to_html/2` to accept AST as input\n  * Added examples directory to show how to use the new APIs","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.18 (2024-07-13) - Changelog","doc":"","ref":"changelog.html#0-1-18-2024-07-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Bump comrak from [0.24.1 to 0.26.0](https://github.com/kivikakk/comrak/blob/9f4d391abe2857031f993c4cdddf1ebba7cdbc7d/changelog.txt#L1-L60)\n  * Add new `extension` options: underline, spoiler, greentext\n  * Add new `render` options: experimental_inline_sourcepos, escaped_char_spans, ignore_setext, ignore_empty_links, gfm_quirks, prefer_fenced","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.17 (2024-06-19) - Changelog","doc":"","ref":"changelog.html#0-1-17-2024-06-19"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Relax minimum required Elixir version to 1.13\n  * Bump comrak to 0.24.1\n  * Bump ammonia to 4.0\n  * Add new `extension` options: multiline_block_quotes, math_dollars, math_code, shortcodes, wikilinks_title_after_pipe, wikilinks_title_before_pipe\n  * Add new `render` option: escaped_char_spans\n  * Add new option `features.syntax_highlight_inline_style` to control whether to embed inline styles or not. Default is `true`.","ref":"changelog.html#enhancements"},{"type":"extras","title":"Changed - Changelog","doc":"* Build binaries on MacOS 12","ref":"changelog.html#changed"},{"type":"extras","title":"0.1.16 (2024-04-29) - Changelog","doc":"","ref":"changelog.html#0-1-16-2024-04-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Added language `objc` to syntax highlighter.","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.15 (2024-04-16) - Changelog","doc":"","ref":"changelog.html#0-1-15-2024-04-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update rustler to `~> 0.32`\n  * Update rustler_precompiled `~> 0.7`\n  * Added legacy targets","ref":"changelog.html#enhancements"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* Minimum required Elixir version is now 1.14\n  * Removed target `arm-unknown-linux-gnueabihf`\n  * Removed target `riscv64gc-unknown-linux-gnu`","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"0.1.14 (2024-04-11) - Changelog","doc":"","ref":"changelog.html#0-1-14-2024-04-11"},{"type":"extras","title":"Backwards incompatible changes - Changelog","doc":"* [Syntax Highlight] Renamed parent ` ` class from `autumn-highlight` to `autumn-hl`\n  * [Syntax Highlight] Added prefix `ahl-` to each scope class","ref":"changelog.html#backwards-incompatible-changes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update autumn to 0.2.2 (#33)\n  * Update comrak to 0.20.0 (#27) - @supernintendo","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.13 (2023-11-20) - Changelog","doc":"","ref":"changelog.html#0-1-13-2023-11-20"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update autumn themes to add base16-tomorrow and base16-tomorrow-night - @paradox460","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.12 (2023-11-06) - Changelog","doc":"","ref":"changelog.html#0-1-12-2023-11-06"},{"type":"extras","title":"Fixes - Changelog","doc":"* Comrak docs links","ref":"changelog.html#fixes"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add languages: jsx, tsx, vim\n  * Bump injket v0.10.2","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.11 (2023-10-25) - Changelog","doc":"","ref":"changelog.html#0-1-11-2023-10-25"},{"type":"extras","title":"Fixes - Changelog","doc":"* Syntax highlighting - remove newlines to avoid formatting incorrectly","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.10 (2023-10-24) - Changelog","doc":"","ref":"changelog.html#0-1-10-2023-10-24"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add translate=\"no\" attr in ` ` tag","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"* Fix Javascript syntax highlight\n  * Fix Typescript syntax highlight","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.9 (2023-09-29) - Changelog","doc":"","ref":"changelog.html#0-1-9-2023-09-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Add logo\n  * Add icon","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.8 (2023-09-29) - Changelog","doc":"","ref":"changelog.html#0-1-8-2023-09-29"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Fallback to plain text on invalid language\n  * Syntax highlight injections\n  * Add samples","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.7 (2023-09-27) - Changelog","doc":"","ref":"changelog.html#0-1-7-2023-09-27"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Syntax highlight code using tree-sitter and helix editor themes. Use https://github.com/leandrocp/autumn/tree/main/native/inkjet_nif under the hood.","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.6 (2023-09-14) - Changelog","doc":"","ref":"changelog.html#0-1-6-2023-09-14"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Load extra themes and syntaxes with https://crates.io/crates/two-face","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.5 (2023-09-12) - Changelog","doc":"","ref":"changelog.html#0-1-5-2023-09-12"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Sanitize output with https://crates.io/crates/ammonia\n  * Syntax Highlight with https://crates.io/crates/syntect","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.4 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-4-2023-09-11"},{"type":"extras","title":"Fixes - Changelog","doc":"* Compile on Ubuntu 20 to fix libc version mismatch","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.3 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-3-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* NIF version 2.15","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.2 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-2-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Guard markdown arg","ref":"changelog.html#enhancements"},{"type":"extras","title":"Fixes - Changelog","doc":"* specs","ref":"changelog.html#fixes"},{"type":"extras","title":"0.1.1 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-1-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* Update Rust to edition 2021\n  * Add `@spec` to public functions","ref":"changelog.html#enhancements"},{"type":"extras","title":"0.1.0 (2023-09-11) - Changelog","doc":"","ref":"changelog.html#0-1-0-2023-09-11"},{"type":"extras","title":"Enhancements - Changelog","doc":"* `MDEx.to_html/1` to convert Markdown to HTML using default options.\n  * `MDEx.to_html/2` to convert Markdown to HTML using custom options.","ref":"changelog.html#enhancements"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}